// ReadPCMICA.cpp
// 720020-1143x
//
// November 1, 2002
// Applied Data Systems
// ctacke
//
// This application is designed as an example to demonstrate the
// use of the ADSCF2 PCMCIA/Compact Flash bus driver (20-1311x).  
// 
// ADSCF2 is provided to allow direct access to the PCMCIA/CF bus for
// additional or external 16-bit hardware access.
//
// This specific test generates a large block of random data.  It then 
// writes this data to the memory space of the PCMCIA slot (to a 
// physical SRAM card in our case), reads the data back in, then 
// compares the written and read data to ensure integrity.
//
// The sample continually cycles, modifying the MECR speed for the
// memory space, gradually getting faster to determine how fast the
// bus can be reliably used.
//
// This application can easily be modified to use the more-easily accessed
// CF bus of the Bitsy by simply changing the occurrences of the PCMCIA 
// SLOT enum to CF
//
// Feb 7, 03 - ctacke
//   - added test procedure for testing reads only by reading first BLOCK_SIZE bytes
//     of attribute space on CF card
//
// Mar 10, 03 - ctacke
//   720020-11431: renumbered and separated test code from ADSCF2 driver code
//
// Mar 24, 03 - ctacke
//   720020-11432: Added constants and code for reading ethernet chip info
//

#include "windows.h"
#include "adscf2.h"

#define BLOCK_SIZE						0x0010
#define ITERATIONS						1000
#define SMC_91C111_CHIPID				9
#define SMC_91C111_BASE					0x300
#define SMC_91C111_BANKSEL_OFFSET		0xE
#define SMC_91C111_REV_OFFSET			0xA

LPVOID   pMem = NULL;
DWORD	 start, elapsed;
volatile ushort	inbuffer[BLOCK_SIZE];
volatile ushort	holdbuffer[BLOCK_SIZE];
volatile ushort	outbuffer[BLOCK_SIZE];
int	     min_left;

ULONG EthernetDetectThread(void *arg)
{
	USHORT data;
	UCHAR  chip;
	UCHAR  rev;

	// Map our base address for SLOT1 Attribute space
	pMem = MapAddress(SLOT1_ATTR_ADDR);

	// select bank 3
	data = 3;
	WriteData16(pMem, (SMC_91C111_BASE + SMC_91C111_BANKSEL_OFFSET), &data, 1);

	// read chip revision data
	ReadData16(pMem, (SMC_91C111_BASE + SMC_91C111_REV_OFFSET), &data, 1);

	// mask out chip ID and revision

    rev = (UCHAR)(data & 0x000f);    
    chip = (UCHAR)((data >> 0x4) & 0xf);
    
    RETAILMSG(TRUE,(_T("Ethernet chip detected.\r\nChip ID: %i "), chip));
	if(chip == SMC_91C111_CHIPID)
    {
		RETAILMSG(TRUE,(_T("(SMSC 91C11) ")));
    }

    RETAILMSG(TRUE,(_T("Revision: %i\r\n"), rev));

	// app will stop withing next 60 seconds
	min_left = 0;

	return 0;
}

ULONG ReadWriteTestThread(void *arg)
{
	int		i;
	int		ret;
	BOOL	failed = FALSE;
	BOOL	fastmode = TRUE;

	// turn on slot
	BusPower(PCMCIA, BITSY_5V);

	// fill a test buffer
	srand(GetTickCount());
	RETAILMSG(1, (_T("Data:\r\n")));
	for(i = 1 ; i <= BLOCK_SIZE ; i++)
	{
		outbuffer[i - 1] = (rand() % 32768);

		RETAILMSG(1, (_T("0x%04x "), outbuffer[i - 1]));
		if((i % 10) == 0)
		{
			RETAILMSG(1, (_T("\r\n")));
		}
	}
	RETAILMSG(1, (_T("\r\n\n")));

	// Map our base address for SLOT0 Memory space
	pMem = MapAddress(SLOT0_MEM_ADDR);

	while(true)
	{
		SetFASTMode(PCMCIA, fastmode);
		RETAILMSG(1, (_T("FAST = %d:\r\n"), fastmode));
		
		BusReset(PCMCIA, 1000);

		for(BYTE timing = 31 ; timing >= 0 ; timing--)
		{
			// set speed
			SetMemorySpeed(PCMCIA, timing);

			Sleep(100);
				
			RETAILMSG(1, (TEXT("Writing...\r\n")));

			start = GetTickCount();

			for(int p = 0 ; p < ITERATIONS ; p++)
			{
				// write data - buffer is in bytes, hence the /2
				WriteData16(pMem, 0, (USHORT *)outbuffer, BLOCK_SIZE / 2);

				// read it back - buffer is in bytes, hence the /2
				ReadData16(pMem, 0, (USHORT *)inbuffer, BLOCK_SIZE / 2);

				// compare it
				ret = memcmp((void *)inbuffer, (void *)outbuffer, BLOCK_SIZE);
				
				if(ret != 0)
				{
					RETAILMSG(1, (_T("Write failure\r\n")));
					// show what we wrote
					RETAILMSG(1, (_T(" W>> ")));
					for(i = 1 ; i <= 8 ; i++)
					{
						RETAILMSG(1, (_T("0x%02x "), outbuffer[i - 1]));
					}
					RETAILMSG(1, (_T("\r\n")));

					// show what we read
					RETAILMSG(1, (_T(" R>> ")));
					for(i = 1 ; i <= 8 ; i++)
					{
						RETAILMSG(1, (_T("0x%02x "), inbuffer[i - 1]));
					}
					RETAILMSG(1, (_T("\r\n\n")));
					
					failed = TRUE;

					break;
				}

			}

			if(!failed)
			{
				elapsed = GetTickCount() - start;

				RETAILMSG(1, (_T("Successful write\r\n")));

				RETAILMSG(1, (TEXT("MECR %02d, Time elapsed %d, # ops: %d*%d (%d k)\r\n"),
							  timing, elapsed, ITERATIONS, BLOCK_SIZE, (int)(float(ITERATIONS*BLOCK_SIZE)/1000+0.5)));
				if (elapsed > 0)
  					RETAILMSG(1, (TEXT("  Access Speed = %d kHz\r\n\n"), (int)((float)(ITERATIONS*BLOCK_SIZE)/(float)elapsed+0.5)));
			}
			else
			{
				failed = FALSE;
			}
		}

		// toggle FAST mode
		fastmode = !fastmode;
	}
}

ULONG ReadOnlyTestThread(void *arg)
{
	int		i;
	int		ret;
	BOOL	failed = FALSE;
	BOOL	fastmode = TRUE;
	BYTE	mecr = 31;

	// turn on slot
	BusPower(CF, BITSY_3V);

	// Map our base address for SLOT1 Attribute space
	pMem = MapAddress(SLOT1_ATTR_ADDR);

	BusReset(CF, 1);
	SetMemorySpeed(CF, mecr);

	while(true)
	{	
		// power the bus
		BusPower(CF, BITSY_3V);

		// assert the reset line
		BusReset(CF, 1);
	
		// wait for card to become ready
		// for the ATA card I tested it was ~5ms
		Sleep(6);

		RETAILMSG(1, (TEXT("Reading...\r\n")));

		start = GetTickCount();

		for(int p = 0 ; p < ITERATIONS ; p++)
		{
			// read data - buffer is in bytes, hence the /2
			ReadData16(pMem, 0, (USHORT *)inbuffer, BLOCK_SIZE / 2);

			// compare it to held data
			ret = memcmp((void *)inbuffer, (void *)holdbuffer, BLOCK_SIZE);
			
			if(ret != 0)
			{
				RETAILMSG(1, (_T("inconsistent read!\r\n")));

				// show what we last read
				RETAILMSG(1, (_T(" LAST>> ")));
				for(i = 1 ; i <= 8 ; i++)
				{
					RETAILMSG(1, (_T("0x%04x "), holdbuffer[i - 1]));
				}
				RETAILMSG(1, (_T("\r\n")));

				// show what we read this time
				RETAILMSG(1, (_T(" THIS>> ")));
				for(i = 1 ; i <= 8 ; i++)
				{
					RETAILMSG(1, (_T("0x%04x "), inbuffer[i - 1]));
				}
				RETAILMSG(1, (_T("\r\n\n")));
				
				failed = TRUE;

				// transfer what we read to the hold buffer
				memcpy((void *)holdbuffer, (void *)inbuffer, BLOCK_SIZE);
			} // if(ret != 0)
		} // for(int p = 0 ; p < ITERATIONS ; p++)

		if(!failed)
		{
			elapsed = GetTickCount() - start;

			RETAILMSG(1, (_T("consistent read\r\n")));

			RETAILMSG(1, (TEXT("MECR %02d, Time elapsed %d, # ops: %d*%d (%d k)\r\n"),
						  mecr, elapsed, ITERATIONS, BLOCK_SIZE, (int)(float(ITERATIONS*BLOCK_SIZE)/1000+0.5)));
			if (elapsed > 0)
  				RETAILMSG(1, (TEXT("  Access Speed = %d kHz\r\n\n"), (int)((float)(ITERATIONS*BLOCK_SIZE)/(float)elapsed+0.5)));
		}
		else
		{
			failed = FALSE;
		}
	} // while(true)
}

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	DWORD threadID;
	HANDLE	hThread;

//	hThread = CreateThread(NULL, NULL, ReadWriteTestThread, NULL, 0, &threadID);
//	hThread = CreateThread(NULL, NULL, ReadOnlyTestThread, NULL, 0, &threadID);
	hThread = CreateThread(NULL, NULL, EthernetDetectThread, NULL, 0, &threadID);

	CeSetThreadPriority(hThread, 50);

	// run for five minutes, max
	for (min_left = 5 ; min_left > 0 ; min_left--)
	{
		RETAILMSG (TRUE, (TEXT("\r\n---- %d minutes left in test ----\r\n\n"),min_left));
		Sleep(60*1000);
	}

	ReleaseAddress(pMem);
	FreeResources();

	RETAILMSG (TRUE, (TEXT("\r\n---- Test complete  ----\r\n")));
	return 0;

}
